@@ -7,6 +7,8 @@ from .util import singleton
 from .actuator import Namespace
 from .actuator import commands
 from .actuator import add_command
+from .actuator import end_command
+from .actuator import pre_command
 from .actuator import run_command
 
 from .parser import argp
