@@ -19,19 +19,35 @@ from .util import singleton
 
 class add_command:
     '''
-    Define command-line arguments.
+    Define a new command-line node.
 
     For example:
 
-    from xarg import add_command\n
-    from xarg import argp\n
+    >>> from xarg import add_command\n
+    >>> from xarg import argp\n
 
-    @add_command("example")\n
-    def cmd(_arg: argp):\n
-        argp.add_opt_on("-t", "--test")\n
+    >>> @add_command("example")\n
+    >>> def cmd(_arg: argp):\n
+    >>>     argp.add_opt_on("-t", "--test")\n
     '''
 
     def __init__(self, name: str, **kwargs):
+        '''
+        @param name: Node name
+        @type name: str
+
+        @param description: Text to display before the argument help
+        @type description: str (by default, no text)
+
+        @param epilog: Text to display after the argument help
+        @type epilog: str (by default, no text)
+
+        @param help: Help message as a subcommand
+        @type help: str
+
+        @param add_help: Add a -h/--help option to the node
+        @type add_help: bool (default: True)
+        '''
         if "help" in kwargs and "description" not in kwargs:
             kwargs["description"] = kwargs["help"]
         if "description" in kwargs and "help" not in kwargs:
@@ -44,26 +60,26 @@ class add_command:
         self.__subs: Optional[Tuple[add_command, ...]] = None
 
     def __call__(self, cmd_func: Callable[[argp], None]):
-        self.__func = cmd_func
+        self.__func: Callable[[argp], None] = cmd_func
         return self
 
     @property
-    def func(self):
+    def func(self) -> Callable[[argp], None]:
         return self.__func
 
     @property
-    def name(self):
+    def name(self) -> str:
         return self.__name
 
     @property
-    def root(self):
+    def root(self) -> "add_command":
         root = self.__prev
         while root.prev != root:
             root = root.prev
         return root
 
     @property
-    def prev(self):
+    def prev(self) -> "add_command":
         return self.__prev
 
     @prev.setter
@@ -72,15 +88,15 @@ class add_command:
         self.__prev = value
 
     @property
-    def cmds(self):
+    def cmds(self) -> "commands":
         return self.__cmds
 
     @property
-    def options(self):
+    def options(self) -> Dict[str, Any]:
         return self.__options
 
     @property
-    def bind(self):
+    def bind(self) -> Optional["run_command"]:
         return self.__bind
 
     @bind.setter
@@ -89,86 +105,225 @@ class add_command:
         self.__bind = value
 
     @property
-    def subs(self):
+    def subs(self) -> Optional[Tuple["add_command", ...]]:
         return self.__subs
 
     @subs.setter
     def subs(self, value: Tuple["add_command", ...]):
+        assert isinstance(value, Tuple)
+        for sub in value:
+            assert isinstance(sub, add_command)
         self.__subs = value
 
     @property
-    def sub_dest(self):
-        return f"__sub_{self.name}_dest__"
+    def sub_dest(self) -> str:
+        node: add_command = self
+        subs: List[str] = [self.name]
+        while node.prev is not node:
+            node = node.prev
+            subs.insert(0, node.name)
+        name = "_".join(subs)
+        return f"__sub_dest_{name}__"
 
 
 class run_command:
     '''
-    Bind command-line arguments and subcommands, define callback functions.
+    Define the main callback function, and bind it to a node and subcommands.
 
     For example:
 
-    from xarg import argp\n
-    from xarg import run_command\n
+    >>> from xarg import commands\n
+    >>> from xarg import run_command\n
 
-    @run_command(cmd, cmd_get, cmd_set)\n
-    def run(cmds: commands) -> int:\n
-        return 0\n
+    >>> @run_command(cmd, cmd_get, cmd_set)\n
+    >>> def run(cmds: commands) -> int:\n
+    >>>     return 0\n
     '''
 
-    def __init__(self, cmd_bind: add_command, *subs: add_command):
-        assert isinstance(cmd_bind, add_command)
-        for sub in subs:
-            sub.prev = cmd_bind
+    def __init__(self, cmd_bind: add_command, *sub_cmds: add_command,
+                 skip: bool = False):
+        '''
+        @param cmd_bind: Bind to a root command node
+        @type name: add_command
 
+        @param *sub_cmds: All required subcommands
+        @type *sub_cmds: add_command
+
+        @param skip: This node (run_command, pre_command and end_command)
+        does not run when a subcommand is specified, run this node without
+        any subcommands
+        @type skip: bool (default: False)
+        '''
+        assert isinstance(cmd_bind, add_command)
+        assert isinstance(skip, bool)
         cmd_bind.bind = self
-        cmd_bind.subs = subs
+        cmd_bind.subs = sub_cmds
+        for sub in sub_cmds:
+            sub.prev = cmd_bind
         commands().root = cmd_bind.root
+        self.__skip: bool = skip
         self.__bind: add_command = cmd_bind
+        self.__prep: Optional["pre_command"] = None
+        self.__done: Optional["end_command"] = None
 
     def __call__(self, run_func: Callable[["commands"], int]):
-        self.__func = run_func
+        self.__func: Callable[["commands"], int] = run_func
         return self
 
     @property
-    def func(self):
+    def func(self) -> Callable[["commands"], int]:
         return self.__func
 
     @property
     def bind(self) -> add_command:
         return self.__bind
 
+    @property
+    def prep(self) -> Optional["pre_command"]:
+        return self.__prep
+
+    @prep.setter
+    def prep(self, value: "pre_command"):
+        assert isinstance(value, pre_command)
+        self.__prep = value
+
+    @property
+    def done(self) -> Optional["end_command"]:
+        return self.__done
+
+    @done.setter
+    def done(self, value: "end_command"):
+        assert isinstance(value, end_command)
+        self.__done = value
+
+    @property
+    def skip(self) -> bool:
+        return self.__skip
+
+
+class pre_command:
+    '''
+    Define prepare callback function, and bind it with main callback.
+
+    For example:
+
+    >>> from xarg import commands\n
+    >>> from xarg import pre_command\n
+    >>> from xarg import run_command\n
+
+    >>> @run_command(cmd, cmd_get, cmd_set)\n
+    >>> def run(cmds: commands) -> int:\n
+    >>>     return 0\n
+
+    >>> @pre_command(run)\n
+    >>> def pre(cmds: commands) -> int:\n
+    >>>     return 0\n
+    '''
+
+    def __init__(self, run_bind: run_command):
+        '''
+        @param cmd_bind: Bind to a root command node
+        @type name: add_command
+        '''
+        assert isinstance(run_bind, run_command)
+        run_bind.prep = self
+        self.__main: run_command = run_bind
+
+    def __call__(self, run_func: Callable[["commands"], int]):
+        self.__func: Callable[["commands"], int] = run_func
+        return self
+
+    @property
+    def func(self) -> Callable[["commands"], int]:
+        return self.__func
+
+    @property
+    def main(self) -> run_command:
+        return self.__main
+
+
+class end_command:
+    '''
+    Define purge callback function, and bind it with main callback.
+
+    For example:
+
+    >>> from xarg import commands\n
+    >>> from xarg import end_command\n
+    >>> from xarg import run_command\n
+
+    >>> @run_command(cmd, cmd_get, cmd_set)\n
+    >>> def run(cmds: commands) -> int:\n
+    >>>     return 0\n
+
+    >>> @end_command(run)\n
+    >>> def end(cmds: commands) -> int:\n
+    >>>     return 0\n
+    '''
+
+    def __init__(self, run_bind: run_command):
+        '''
+        @param cmd_bind: Bind to a root command node
+        @type name: add_command
+        '''
+        assert isinstance(run_bind, run_command)
+        run_bind.done = self
+        self.__main: run_command = run_bind
+
+    def __call__(self, run_func: Callable[["commands"], int]):
+        self.__func: Callable[["commands"], int] = run_func
+        return self
+
+    @property
+    def func(self) -> Callable[["commands"], int]:
+        return self.__func
+
+    @property
+    def main(self) -> run_command:
+        return self.__main
+
 
 @singleton
 class commands:
     '''
     Singleton command-line tool based on argparse.
 
-    Define and bind callback functions before calling run() or parse().
+    Define and bind all callback functions before calling run() or parse().
 
     For example:
 
-    from typing import Optional\n
-    from typing import Sequence\n
+    >>> from typing import Optional\n
+    >>> from typing import Sequence\n
 
-    from xarg import add_command\n
-    from xarg import argp\n
-    from xarg import commands\n
-    from xarg import run_command\n
+    >>> from xarg import add_command\n
+    >>> from xarg import argp\n
+    >>> from xarg import commands\n
+    >>> from xarg import end_command\n
+    >>> from xarg import pre_command\n
+    >>> from xarg import run_command\n
 
-    @add_command("example")\n
-    def cmd(_arg: argp):\n
-        argp.add_opt_on("-t", "--test")\n
+    >>> @add_command("example")\n
+    >>> def cmd(_arg: argp):\n
+    >>>     argp.add_opt_on("-t", "--test")\n
 
-    @run_command(cmd, cmd_get, cmd_set)\n
-    def run(cmds: commands) -> int:\n
-        return 0\n
+    >>> @run_command(cmd, cmd_get, cmd_set)\n
+    >>> def run(cmds: commands) -> int:\n
+    >>>     return 0\n
 
-    def main(argv: Optional[Sequence[str]] = None) -> int:\n
-        return commands().run(\n
-            root=cmd,\n
-            argv=argv,\n
-            prog="xarg-example",\n
-            description="Simple command-line tool based on argparse.")\n
+    >>> @pre_command(run)\n
+    >>> def pre(cmds: commands) -> int:\n
+    >>>     return 0\n
+
+    >>> @end_command(run)\n
+    >>> def end(cmds: commands) -> int:\n
+    >>>     return 0\n
+
+    >>> def main(argv: Optional[Sequence[str]] = None) -> int:\n
+    >>>     return commands().run(\n
+    >>>         root=cmd,\n
+    >>>         argv=argv,\n
+    >>>         prog="xarg-example",\n
+    >>>         description="Simple command-line tool based on argparse.")\n
     '''
 
     LOGGER_ARGUMENT_GROUP = "logger options"
@@ -241,14 +396,14 @@ class commands:
         '''
         return logging.getLogger(self.prog)
 
-    def stdout(self, context):
+    def stdout(self, context: Any):
         '''
         Output string to sys.stdout.
         '''
         sys.stdout.write(f"{context}\n")
         sys.stdout.flush()
 
-    def stderr(self, context):
+    def stderr(self, context: Any):
         '''
         Output string to sys.stderr.
         '''
@@ -411,11 +566,12 @@ class commands:
         _sub = arg_root.add_subparsers(dest=cmd_root.sub_dest)
         for sub in cmd_root.subs:
             assert isinstance(sub, add_command)
+            options = sub.options.copy()
             for key, value in kwargs.items():
-                sub.options.setdefault(key, value)
-            sub.options.setdefault("epilog", arg_root.epilog)
-            sub.options.setdefault("prev_parser", arg_root)
-            _arg: argp = _sub.add_parser(sub.name, **sub.options)
+                options.setdefault(key, value)
+            options.setdefault("epilog", arg_root.epilog)
+            options.setdefault("prev_parser", arg_root)
+            _arg: argp = _sub.add_parser(sub.name, **options)
             self.__add_parser(_map, _arg, sub)
 
     def __add_option(self, _map: Dict[add_command, argp]):
@@ -447,13 +603,34 @@ class commands:
         self.args = args
         return self.args
 
+    def has_sub(self, root: add_command,
+                args: Optional[Namespace] = None) -> bool:
+        '''
+        If the root command node has any subcommand nodes, return true.
+
+        @param root: Command node
+        @type root: add_command
+
+        @param args: Command arguments
+        @type args: Namespace or None (default self.args if None is specified)
+
+        @return: bool
+        '''
+        if args is None:
+            args = self.args
+        assert isinstance(root, add_command)
+        assert isinstance(args, Namespace)
+        return isinstance(getattr(args, root.sub_dest), str)\
+            if hasattr(args, root.sub_dest) else False
+
     def __run(self, args: Namespace, root: add_command) -> int:
         assert isinstance(root, add_command)
         assert isinstance(root.bind, run_command)
 
-        ret = root.bind.func(self)
-        if ret != 0 and ret is not None:
-            return ret
+        if not root.bind.skip or not self.has_sub(root, args):
+            ret = root.bind.func(self)
+            if ret != 0 and ret is not None:
+                return ret
 
         if hasattr(args, root.sub_dest):
             sub_dest = getattr(args, root.sub_dest)
@@ -462,7 +639,39 @@ class commands:
                 for sub in root.subs:
                     assert isinstance(sub, add_command)
                     if sub.name == sub_dest:
-                        return self.__run(args, sub)
+                        ret = self.__run(args, sub)
+                        if ret != 0 and ret is not None:
+                            return ret
+
+        done = root.bind.done
+        if done is not None:
+            assert isinstance(done, end_command)
+            if not root.bind.skip or not self.has_sub(root, args):
+                ret = done.func(self)  # purge
+                if ret != 0 and ret is not None:
+                    return ret
+        return 0
+
+    def __pre(self, args: Namespace, root: add_command) -> int:
+        assert isinstance(root, add_command)
+        assert isinstance(root.bind, run_command)
+
+        prep = root.bind.prep
+        if prep is not None:
+            assert isinstance(prep, pre_command)
+            if not root.bind.skip or not self.has_sub(root, args):
+                ret = prep.func(self)
+                if ret != 0 and ret is not None:
+                    return ret
+
+        if hasattr(args, root.sub_dest):
+            sub_dest = getattr(args, root.sub_dest)
+            if isinstance(sub_dest, str):
+                assert isinstance(root.subs, (list, tuple))
+                for sub in root.subs:
+                    assert isinstance(sub, add_command)
+                    if sub.name == sub_dest:
+                        return self.__pre(args, sub)
         return 0
 
     def run(self,
@@ -490,6 +699,9 @@ class commands:
                 # items are debug level only, except for errors.
                 self.logger.debug(f"version: {version}")
 
+            ret = self.__pre(args, root)
+            if ret != 0 and ret is not None:
+                return ret
             return self.__run(args, root)
         except KeyboardInterrupt:
             return EINTR
