@@ -1,4 +1,3 @@
-#!/usr/bin/python3
 # coding:utf-8
 
 from base64 import b16decode
@@ -15,15 +14,17 @@ from typing import Set
 from typing import Tuple
 from typing import Union
 
+from pip._internal.commands.show import _PackageInfo
+from pip._internal.commands.show import search_packages_info
 from tabulate import tabulate
 
 from .actuator import add_command
 from .actuator import commands
 from .actuator import run_command
 from .parser import argp
-from .util import URL_PROG
-from .util import __package_name__
+from .util import __name__
 from .util import __prog_complete__
+from .util import __url_home__
 from .util import __version__
 from .util import singleton
 
@@ -83,78 +84,28 @@ def list_bash() -> Set[str]:
     return cmds
 
 
-class package_info:
-
-    def __init__(self, name: str):
-        self.__pkg_infos: Dict[str, str] = {}
-        self.__pkg_files: set[str] = set()
-        self.__have_file = False
-        with os.popen(f"pip show --files {name}") as fd:
-            for line in fd.readlines():
-                if not self.__have_file:
-                    title, value = line.split(":", 1)
-                    if title != "Files":
-                        self.__pkg_infos[title] = value.strip()
-                    else:
-                        self.__location = self.__pkg_infos["Location"]
-                        self.__have_file = True
-                else:
-                    path = os.path.join(self.location, line.strip())
-                    abspath = os.path.abspath(path)
-                    self.__pkg_files.add(abspath)
-                    if "entry_points.txt" in abspath:
-                        self.__entry_points = abspath
-
-    @property
-    def name(self) -> str:
-        return self.__pkg_infos["Name"]
-
-    @property
-    def version(self) -> str:
-        return self.__pkg_infos["Version"]
-
-    @property
-    def location(self) -> str:
-        return self.__location
-
-    @property
-    def requires(self) -> Set[str]:
-        return {i.strip() for i in self.__pkg_infos["Requires"].split(",")}
-
-    @property
-    def required_by(self) -> Set[str]:
-        return {i.strip() for i in self.__pkg_infos["Required-by"].split(",")}
-
-    @property
-    def files(self) -> Set[str]:
-        return self.__pkg_files
-
-    @property
-    def console_scripts(self) -> Set[str]:
-        try:
-            config = ConfigParser()
-            config.read(self.__entry_points)
-            return {i for i in config["console_scripts"]}
-        except Exception:
-            return set()
-
-
 @singleton
 class collections:
 
     def __init__(self):
-        info = package_info(__package_name__)
-        self.__pkgs: Set[str] = {i for i in info.required_by}
-        self.__cmds: Set[str] = {__prog_complete__}
-        for pkg in self.__pkgs:
-            pkgi = package_info(pkg)
-            for cmd in pkgi.console_scripts:
-                self.__cmds.add(cmd)
+        self.__cmds: Set[str] = set()
+        for _pkg in {"argcomplete", __name__}:
+            for _req in {i for i in self.get_package_info(_pkg).required_by}:
+                config = ConfigParser()
+                package_info = self.get_package_info(_req)
+                config.read_string(os.linesep.join(package_info.entry_points))
+                if config.has_section("console_scripts"):
+                    for _cmd in config["console_scripts"]:
+                        self.__cmds.add(_cmd)
 
     @property
     def cmds(self) -> Iterable[str]:
         return iter(self.__cmds)
 
+    @classmethod
+    def get_package_info(cls, package_name: str) -> _PackageInfo:
+        return [i for i in search_packages_info([package_name])][0]
+
 
 @add_command("enable", help="Enable completion.")
 def add_cmd_enable(_arg: argp):
@@ -283,4 +234,4 @@ def main(argv: Optional[Sequence[str]] = None) -> int:
         root=add_cmd,
         argv=argv,
         description="Tab completion management.",
-        epilog=f"For more, please visit {URL_PROG}.")
+        epilog=f"For more, please visit {__url_home__}.")
